/*  FreeJ
 *  (c) Copyright 2001 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream>
#include <assert.h>
#include <context.h>
#include <keyboard.h>
#include <v4l.h>
#include <utils.h>
#include <config.h>
#include <gui_layer.h>
#include "filters/filter-list.h"

#define W 400
#define H 300
#define D 16

/* this is the output context (screen) */
Context *screen = NULL;

/* this is a layer: actually only v4l is implemented */
V4lGrabber *grabber = NULL;

/* this is the keyboard listener */
KbdListener *keyb = NULL;

/* those are the pointers to filters 
   they have to be instantiated still */
Strobe *strobe = NULL;
Bitcolor *bitcolor = NULL;
Delaygrab *delaygrab = NULL;
Absdiff *absdiff = NULL;
Rotozoom *rotozoom = NULL;

int main (int argc, char **argv) {
  
  fprintf(stderr,"%s version %s [ http://freej.dyne.org ]\n",PACKAGE,VERSION);
  cerr << "(c)2001 Denis Roio aka jaromil\n";
  set_debug(3);

  /* allocates the context, SDL flags are used directly here */
  screen = new Context(W,H,D,SDL_HWSURFACE|SDL_DOUBLEBUF);
  if(screen->surf==NULL) exit(0);

  /* allocates v4l grabber layer */
  grabber = new V4lGrabber();
  if(!grabber->detect("/dev/video")) {
    delete grabber;
    delete screen;
    exit(0);
  }

  /* init the grabber */
  if(!grabber->init(screen,256,256)) {
    delete grabber;
    delete screen;
    exit(0);
  }
  /* center the layer */
  grabber->x = (Uint16)(W-256)/2;
  grabber->y = (Uint16)(H-256)/2;
  /* --------------------- */

  /* create and init the keyb listener */
  keyb = new KbdListener();
  keyb->init(screen);
  
  /* update the fps counter every 25 frames */
  screen->set_fps_interval(25);

  /* create and initialize the gui */
  GuiLayer gui;
  gui.init(grabber);

  grabber->start();
  screen->start();
  keyb->start();
  gui.start();

  while(!keyb->quit) SDL_Delay(3000);
  keyb->join();

  gui.quit = true;
  gui.join();

  screen->quit = true;
  screen->join();

  if(keyb!=NULL)
    delete keyb;
  if(screen!=NULL)
    delete screen;

  exit(1);

}
