#include <stdio.h>
#include <iostream.h>
#include <utils.h>
#include <config.h>
#include <filter.h>
#include <gui_callbacks.h>
#include "filters/filter-list.h"

static GuiLayer *gui = NULL;
static int select = 0;

void cb_init(GuiLayer *init_gui) {
  gui = init_gui;
}

void cb_sync() {
  ((LTI::Object *)gui)->syncwith((LTI::Object *)gui->layer,gui->layer->id);
}

void cb_keypress(Layer *layer, SDL_keysym keysym) {
  cb_sync();
  layer->keypress(&keysym);
} 

void cb_filteradd(Fl_Widget *o, void *data) {
  char tmp[128];
  Filter *filt = NULL;
  int sel = ((Fl_Menu_Button *)o)->value();
  GuiLayer *gui = (GuiLayer *) data;

  switch(sel) {
  case 0:
    filt = new Bitcolor;
    break;
  case 1:
    filt = new Strobe;
    break;
  case 2:
    filt = new Delaygrab;
    break;
  case 3:
    filt = new Absdiff;
    break;
  case 4:
    filt = new Rotozoom;
    break;
  }
  
  /* PARANOIA */
  if(filt==NULL) return;

  cb_sync();
  gui->layer->add_filter(filt);

  snprintf(tmp,128,"[A] [L] %s",filt->name);
  gui->regfilter_browser->add(tmp);
  func("GUI: registered filter %s",filt->name);
}

void cb_browser(Fl_Widget *o, void *data) {
  select = ((Fl_Hold_Browser *)o)->value();
}

void cb_filterdel(Fl_Widget *o, void *data) {
  if(select==0)return;
  GuiLayer *gui = (GuiLayer *) data;

  cb_sync();
  if(!gui->layer->del_filter(select)) return;

  gui->regfilter_browser->remove(select);
  select = 0;
}

void cb_filterup(Fl_Widget *o, void *data) {
  if(select==0) return;
  GuiLayer *gui = (GuiLayer *) data;

  cb_sync();
  if(!gui->layer->moveup_filter(select)) return;

  gui->regfilter_browser->move(select,select-1);  
  select--;  
}

void cb_filterdown(Fl_Widget *o, void *data) {
  if(select==0) return;
  GuiLayer *gui = (GuiLayer *) data;
  
  cb_sync();
  if(!gui->layer->movedown_filter(select)) return;

  gui->regfilter_browser->move(select,select+1);
  select++;
}

void cb_filterclear(Fl_Widget *o, void *data) {
  GuiLayer *gui = (GuiLayer *) data;

  cb_sync();
  gui->layer->clear_filters();

  gui->regfilter_browser->clear();
}

void cb_switchactive(Fl_Widget *o, void *data) {
  Filter *filt;
  char tmp[128];
  if(select==0)return;
  GuiLayer *gui = (GuiLayer *) data;

  //  cb_sync();
  filt = gui->layer->active_filter(select);

  snprintf(tmp,128,"[%s] [%s] %s",
	   (filt->active) ? "A" : " ",
	   (filt->listen) ? "L" : " ",
	   filt->name);
  gui->regfilter_browser->text(select,tmp);
}

void cb_switchlisten(Fl_Widget *o, void *data) {
  Filter *filt;
  char tmp[128];
  if(select==0)return;
  GuiLayer *gui = (GuiLayer *) data;

  //  cb_sync();
  filt = gui->layer->listen_filter(select);

  snprintf(tmp,128,"[%s] [%s] %s",
	   (filt->active) ? "A" : " ",
	   (filt->listen) ? "L" : " ",
	   filt->name);
  gui->regfilter_browser->text(select,tmp);
}

Fl_Window *aboutwin = NULL;
#include <FL/Fl_Box.H>
#include <FL/Fl_Window.H>

inline void cb_about() {
  static char temp[128];
  aboutwin = new Fl_Window(300,350, "FreeJ - free the veejay!");
  aboutwin->color(101);
  
  {
    sprintf(temp,"%s v%s codename \"ETNA\"\nhttp://freej.dyne.org",PACKAGE,VERSION);
    Fl_Box *box = new Fl_Box(35, 15, 250, 30, temp);
    box->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
    box->labelsize(12);
    box->labelfont(1);
  }

  {
    Fl_Box *authors = new Fl_Box(15, 70, 300, 50, "Author:\nDenis Roio aka jaromil <jaromil@dyne.org>");
    authors->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
    authors->labelsize(12);
    authors->labelfont(1);
  }

  {
    Fl_Box *message = new Fl_Box(15,130,280,50,
				 "Please join the FreeJ mailinglist writing a mail to <freej-subscribe@dyne.org> to report bugs and give useful feedback. Thanks in advance for your collaboration.\nIf you are interested in supporting this project for further development read the included README file and contact <freej-team@dyne.org>.\nThis release is a preview of what FreeJ aims to be, still many planned features are missing and you may expect some crashes under some circumstances.");
    message->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
    message->labelsize(11);
    message->labelfont(1);
  }

  {
    Fl_Box *disclaimer = new Fl_Box(15,280,300,50, "FreeJ comes with ABSOLUTELY NO WARRANTY\n This is Free Software, and you are welcome\n to redistribute it under certain conditions\n refer to the Gnu Public License for details");
    disclaimer->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
    disclaimer->labelsize(11);
    disclaimer->labelfont(1);
  }
  
  aboutwin->end();
  aboutwin->show();
}

void cb_showabout(Fl_Widget *o) {
  if(aboutwin!=NULL) {
    if(!aboutwin->shown())
      aboutwin->show();
  } else
    cb_about();
}
