#include <gui_callbacks.h>
#include <gui_layer.h>
#include <utils.h>
#include <config.h>

#define WIDTH 426
#define HEIGHT 267
#define UPDATE_INTERVAL 0.1

Fl_Menu_Item GuiLayer::menu_filteradd[] = {
 {"Bitcolor", 0,  0, 0, 0, 0, 13, 12, 0},
 {"Strobe", 0,  0, 0, 0, 0, 13, 12, 0},
 {"Delaygrab", 0,  0, 0, 0, 0, 13, 12, 0},
 {"Absdiff", 0,  0, 0, 0, 0, 13, 12, 0},
 {"Rotozoom", 0,  0, 0, 0, 0, 13, 12, 0},
 {0}
};
Fl_Menu_Item* GuiLayer::Bitcolor = GuiLayer::menu_filteradd + 0;

GuiLayer::GuiLayer() : Fl_Window(WIDTH,HEIGHT,PACKAGE) {
  quit = false;
  layer = NULL;
  window = make_window(WIDTH,HEIGHT,PACKAGE);

  synchronous.registerObjectName("GuiLayer");
}

void GuiLayer::init(Layer *lay) { 
  layer = lay; 
  cb_init(this);
}

void GuiLayer::run() {
  if(layer==NULL) {
    warning("can't start GUI: no layer has been registered"); return;
  }

  window->show();
  
  while(!quit) {
    if((Fl::wait(UPDATE_INTERVAL)<0.0))// || !window->shown())
      quit = true;
  }

  if(window->shown())
    window->hide();

  notice("GUI exiting");
}

Fl_Window* GuiLayer::make_window(int w, int h, char *title) {
  Fl_Window *win;
  { 
    Fl_Window* o = new Fl_Window(w, h, title);
    win = o;
    o->color(206);
    //    o->selection_color(14);
    o->labelsize(10);
    o->user_data((void*)(this));
    {
      Fl_Hold_Browser* o = regfilter_browser = new Fl_Hold_Browser(10, 40, 160, 220, "FILTER CHAIN");
      o->box(FL_THIN_DOWN_FRAME);
      o->color(215);
      //      o->selection_color(195);
      o->labelsize(10);
      o->textfont(13);
      o->textsize(10);
      o->align(FL_ALIGN_TOP);
      o->callback(cb_browser, this);
    }
    { 
      Fl_Button* o = about = new Fl_Button(0, 0, 425, 25, "Freej 0.2 - video4linux layer");
      o->box(FL_ENGRAVED_BOX);
      o->color(149);
      o->labeltype(FL_ENGRAVED_LABEL);
      o->labelfont(14);
      o->callback(cb_showabout);
    }
    { 
      Fl_Tile* o = new Fl_Tile(175, 40, 245, 95, "filter operations");
      o->box(FL_THIN_DOWN_FRAME);
      o->labelfont(13);
      o->labelsize(12);
      { 
	Fl_Button* o = filterclear = new Fl_Button(210, 105, 95, 25, "clear chain");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labelfont(13);
        o->labelsize(12);
	o->callback(cb_filterclear, this);
      }
      { 
	Fl_Button* o = filteractive = new Fl_Button(210, 75, 95, 25, "activate");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labelfont(13);
        o->labelsize(12);
	o->callback(cb_switchactive,this);
      }
      { 
	Fl_Button* o = filterlisten = new Fl_Button(310, 75, 105, 25, "listen keyb");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labelfont(13);
        o->labelsize(12);
	o->callback(cb_switchlisten,this);
      }
      { 
	Fl_Button* o = filterinfo = new Fl_Button(310, 105, 105, 25, "filter info");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labelfont(13);
        o->labelsize(12);
      }
      { 
	Fl_Button* o = new Fl_Button(180, 75, 25, 25, "@8");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labeltype(FL_SYMBOL_LABEL);
	o->callback(cb_filterup,this);
      }
      { 
	Fl_Button* o = new Fl_Button(180, 105, 25, 25, "@2");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labeltype(FL_SYMBOL_LABEL);
	o->callback(cb_filterdown,this);
      }
      { 
	Fl_Button* o = filterdel = new Fl_Button(310, 45, 105, 25, "remove");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_DOWN_BOX);
        o->labelfont(13);
        o->labelsize(12);
	o->callback(cb_filterdel,this);
      }
      { 
	Fl_Menu_Button* o = filteradd = new Fl_Menu_Button(180, 45, 125, 25, "add");
        o->box(FL_THIN_UP_BOX);
        o->down_box(FL_THIN_UP_BOX);
        o->labelfont(13);
        o->labelsize(12);
        o->textfont(13);
        o->textsize(12);
        o->menu(menu_filteradd);
	o->callback(cb_filteradd, (void *) this);
      }
      o->end();
    }
    o->end();
  }
  return win;
}
