/*  FreeJ
 *  (c) Copyright 2001 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <layer.h>
#include <context.h>
#include <lubrify.h>
#include <gui_callbacks.h>

Layer::~Layer() {
  Filter *tmp, *filt = (Filter *)filters.begin();

  func("Layer::~Layer()");
  
  while(filt!=NULL) {
    tmp = (Filter *)filt->next;
    delete(filt);
    filt = tmp;
  }
}

void Layer::_init() {
  _w = w; _h = h;
  _pitch = pitch;
  _size = size;
  quit = false;
}

void Layer::run() {
  int sel;
  while(!quit) {
    synchronous.select.accept(0, id);

    synchronous.select.waitUntil(1, LTI::Kernel::now());
    sel = synchronous.wait();

    if(!feeded) feed();
  }
}

bool Layer::add_filter(Filter *newfilt) {
  bool res = false;
  if(newfilt==NULL) {
    warning("Layer::add_filter called with an invalid NULL filter");
    syncreply();
    return(res);
  }

  func("Layer::add_filter(%s)",newfilt->name);

  if(!newfilt->bpp_ok(bpp)) {
    warning("%s filter does'nt support %ubpp",newfilt->name,bpp);
    syncreply();
    return(res);
  }

  /* here pointers are used to let filters change the size */
  newfilt->w = &w; newfilt->h = &h;
  newfilt->pitch = &pitch; newfilt->size = &size;
  /* --- not yet fully supported as a possibility for the filters */
  /* --- needs work */

  newfilt->bpp = bpp;
  newfilt->fps = fps; /* this is a pointer from the screen */

  notice("%s filter registered :: w[%u] h[%u] bpp[%u] size[%u]",newfilt->name,w,h,bpp,size);

  if(!newfilt->initialized) {
    res = newfilt->init();
    newfilt->initialized = true;
  } else res = true;

  /* add the filter to the linklist */
  filters.add(newfilt);

  syncreply();
  return(res);
}

bool Layer::del_filter(int sel) {
  func("Layer::del_filter(%u)",sel);
  Filter *filt = (Filter *) filters.pick(sel);
  /* PARANOIA */
  if(filt==NULL) {
    warning("Layer::del_filter - filters.pick(%u) returned a NULL Filter",sel);
    syncreply();
    return(false);
  };

  filters.rem(sel);
  delete(filt);
  syncreply();
  return(true);
}

void Layer::clear_filters() {
  func("Layer::clear_filters()");
  Filter *f = (Filter *)filters.begin();

  while(f!=NULL) {
    filters.rem(1);
    delete(f);
    f = (Filter *)filters.begin();
  }
  syncreply();
}

bool Layer::moveup_filter(int sel) {
  func("Layer::moveup_filter(%u)",sel);
  bool res = filters.moveup(sel);
  syncreply();
  return(res);
}

bool Layer::movedown_filter(int sel) {
  //  func("Layer::movedown_filter(%u)",sel);
  bool res = filters.movedown(sel);
  syncreply();
  return(res);
}

Filter *Layer::active_filter(int sel) {
  //  func("Layer::active_filter(%u)",sel);
  Filter *filt = (Filter *)filters.pick(sel);
  filt->active = !filt->active;
  //  syncreply();
  return(filt);
}

Filter *Layer::listen_filter(int sel) {
  //  func("Layer::listen_filter(%u)",sel);
  Filter *filt = (Filter *)filters.pick(sel);
  filt->listen = !filt->listen;
  //  syncreply();
  return(filt);
}

void Layer::keypress(SDL_keysym *keysym) {
  Filter *filt = (Filter *)filters.begin();
  while(filt!=NULL) {
    if(filt->listen) filt->kbd_input(keysym);
    filt = (Filter *)filt->next;
  }
  syncreply();
}


bool Layer::cafudda() {
  void *res = get_buffer();
  // func("Layer::cafudda()");
  /* restore original size info so that it
     can be changed from the filters */
  w = _w; h = _h; pitch = _pitch; size = _size;

  Filter *filt = (Filter *)filters.begin();

  while(filt!=NULL) {
    if(filt->active) res = filt->process(res);
    filt = (Filter *)filt->next;
  }

  if(pitch==screen->pitch)
    mmxcopy(res,screen->get_surface(),size);
  else
    mmxblit(res,screen->coords(x,y),h,pitch,screen->pitch); 
  /* pitch is width in bytes */

  feeded = false;
  
  syncreply();
  return(true);
}
